#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from __future__ import annotations

import warnings
from gettext import gettext as tr
from typing import TYPE_CHECKING

from holidays.calendars.chinese import KOREAN_CALENDAR
from holidays.calendars.gregorian import (
    JAN,
    FEB,
    MAR,
    APR,
    MAY,
    JUN,
    JUL,
    AUG,
    SEP,
    OCT,
    NOV,
    DEC,
    _timedelta,
)
from holidays.constants import BANK, PUBLIC
from holidays.groups import (
    ChineseCalendarHolidays,
    ChristianHolidays,
    InternationalHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_WORKDAY,
    SUN_TO_NEXT_WORKDAY,
)

if TYPE_CHECKING:
    from datetime import date


class SouthKorea(
    ObservedHolidayBase,
    ChineseCalendarHolidays,
    ChristianHolidays,
    InternationalHolidays,
    StaticHolidays,
):
    """South Korea holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_South_Korea>
        * <https://web.archive.org/web/20240429121214/https://www.law.go.kr/법령/관공서의%20공휴일에%20관한%20규정>
        * [Public Holidays Act (2026 Amendment)](http://archive.today/2026.02.20-080801/https://www.law.go.kr/LSW/lsInfoP.do?lsiSeq=283311&ancYd=20260210%230000)
        * [National Day Act (2014 Amendment)](http://archive.today/2026.02.20-081906/https://www.law.go.kr/LSW//lsInfoP.do?lsiSeq=165523&ancYd=20141230%230000)
        * <https://web.archive.org/web/20250429081641/https://elaw.klri.re.kr/eng_service/lawView.do?lang=ENG&hseq=34678>
        * <https://web.archive.org/web/20250123212346/https://elaw.klri.re.kr/eng_service/%20lawView.do?hseq=38405&lang=ENG>
        * <https://namu.wiki/w/대통령%20선거일>
        * <https://namu.wiki/w/공휴일/대한민국>
        * <https://namu.wiki/w/공휴일/대한민국/역사>
        * <https://namu.wiki/w/대체%20휴일%20제도>
        * [TH localization 1](https://web.archive.org/web/20241217184803/https://overseas.mofa.go.kr/th-th/wpge/m_3133/contents.do)
        * [TH localization 2](https://web.archive.org/web/20200216004120/http://thailand.korean-culture.org:80/th/138/korea/38)

    Checked With:
        * <https://web.archive.org/web/20231202051034/https://publicholidays.co.kr/ko/2020-dates/>
        * <https://web.archive.org/web/20231002172705/https://publicholidays.co.kr/ko/2022-dates/>

    According to Article 3 of the Public Holidays Act:
        * The alternative holiday is the first non-holiday after the holiday.
        * Independence Movement Day, Liberation Day, National Foundation Day,
            Hangul Day, Children's Day, Birthday of the Buddha, and Christmas Day have
            an alternative holiday if they fall on Saturday or Sunday.
        * Korean New Year's Day and Korean Mid-Autumn Day have an alternative holiday if they
            fall on Sunday.
    """

    country = "KR"
    supported_categories = (BANK, PUBLIC)
    default_language = "ko"
    # %s (estimated).
    estimated_label = tr("%s (추정)")
    # Alternative holiday for %s (estimated).
    observed_estimated_label = tr("%s 대체 휴일 (추정)")
    # Alternative holiday for %s.
    observed_label = tr("%s 대체 휴일")
    supported_languages = ("en_US", "ko", "th")
    start_year = 1948

    def __init__(self, *args, **kwargs):
        ChineseCalendarHolidays.__init__(self, calendar=KOREAN_CALENDAR)
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, cls=SouthKoreaStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_WORKDAY)
        kwargs.setdefault("observed_since", 2014)
        super().__init__(*args, **kwargs)

    def _populate_observed(self, dts: set[date], three_day_holidays: dict[date, str]) -> None:
        for dt in sorted(dts.union(three_day_holidays.keys())):
            if not self._is_observed(dt):
                continue
            dt_observed = self._get_observed_date(
                dt, SUN_TO_NEXT_WORKDAY if dt in three_day_holidays else SAT_SUN_TO_NEXT_WORKDAY
            )
            if dt_observed != dt or len(self.get_list(dt)) > 1:
                if dt_observed == dt:
                    dt_observed = self._get_next_workday(dt)
                names = (
                    (three_day_holidays[dt],) if dt in three_day_holidays else self.get_list(dt)
                )
                for name in names:
                    self._add_holiday(self.tr(self.observed_label) % self.tr(name), dt_observed)

    def _populate_public_holidays(self):
        def append_observed(dt: date, since: int):
            if self._year >= since:
                dts_observed.add(dt)

        def add_three_day_holiday(dt: date, name: str):
            name = self.tr(name)
            for dt_alt in (
                # The day preceding %s.
                self._add_holiday(self.tr("%s 전날") % name, _timedelta(dt, -1)),
                dt,
                # The second day of %s.
                self._add_holiday(self.tr("%s 다음날") % name, _timedelta(dt, +1)),
            ):
                three_days_holidays[dt_alt] = name

        dts_observed = set()
        three_days_holidays = {}

        # Fixed Date Holidays.

        # New Year's Day.
        name = tr("신정연휴")
        self._add_new_years_day(name)
        if self._year <= 1998:
            self._add_new_years_day_two(name)
        if self._year <= 1989:
            self._add_new_years_day_three(name)

        if self._year >= 1985:
            name = (
                # Korean New Year.
                tr("설날")
                if self._year >= 1989
                # Folk Day.
                else tr("민속의 날")
            )
            korean_new_year = self._add_chinese_new_years_day(name)
            if self._year >= 1989:
                add_three_day_holiday(korean_new_year, name)

        # Independence Movement Day.
        mar_1 = self._add_holiday_mar_1(tr("삼일절"))
        # mar_1 is used later for Presidential Election Day.
        append_observed(mar_1, 2022)

        if 1949 <= self._year <= 2005 and self._year != 1960:
            # Tree Planting Day.
            self._add_holiday_apr_5(tr("식목일"))

        if self._year >= 1975:
            name = (
                # Buddha's Birthday.
                tr("부처님오신날")
                if self._year >= 2017
                # Buddha's Birthday.
                else tr("석가탄신일")
            )
            append_observed(self._add_chinese_birthday_of_buddha(name), 2023)

        if self._year >= 1975:
            # Children's Day.
            append_observed(self._add_holiday_may_5(tr("어린이날")), 2015)

        if self._year >= 1956:
            # Memorial Day.
            jun_6 = self._add_holiday_jun_6(tr("현충일"))
            # jun_6 is used later for Local Election Day.

        if self._year <= 2007 or self._year >= 2026:
            # Constitution Day.
            self._add_holiday_jul_17(tr("제헌절"))

        # Liberation Day.
        append_observed(self._add_holiday_aug_15(tr("광복절")), 2021)

        if 1976 <= self._year <= 1990:
            # Armed Forces Day.
            self._add_holiday_oct_1(tr("국군의 날"))

        # National Foundation Day.
        append_observed(self._add_holiday_oct_3(tr("개천절")), 2021)

        if self._year <= 1990 or self._year >= 2013:
            # Hangul Day.
            append_observed(self._add_holiday_oct_9(tr("한글날")), 2021)

        if 1950 <= self._year <= 1975:
            # United Nations Day.
            self._add_united_nations_day(tr("국제연합일"))

        # Chuseok.
        name = tr("추석")
        chuseok = self._add_mid_autumn_festival(name)
        if 1986 <= self._year <= 1988:
            self._add_mid_autumn_festival_day_two(self.tr("%s 다음날") % self.tr(name))
        elif self._year >= 1989:
            add_three_day_holiday(chuseok, name)

        # Christmas Day.
        append_observed(self._add_christmas_day(tr("기독탄신일")), 2023)

        # Election Days since Sep 2006; excluding the 2017, 2025 Special Presidential Election Day.

        # Based on Article 34 of the Public Official Election Act.
        # (1) The election day for each election to be held at the expiration of the term shall
        #     be as follows: <Amended by Act No. 5508, Feb. 6, 1998; Act No. 7189, Mar. 12, 2004>
        #       1. The presidential election shall be held on the first Wednesday from the 70th
        #          day before the expiration of the term of office;
        #       2. The election of National Assembly members shall be held on the first Wednesday
        #          from the 50th day before the expiration of the term of office;
        #       3. The election of local council members and the head of each local government
        #          shall be held on the first Wednesday from the 30th day before the expiration
        #          of the term of office.
        # (2) Where the election day as provided in paragraph (1) falls on a folk festival day or
        #     legal holiday closely related with the lives of the people or the day preceding or
        #     following the election day is a legal holiday, the election shall be held on the
        #     Wednesday of the following week. <Amended by Act No. 7189, Mar. 12, 2004>

        # National Assembly Election Day.
        name = tr("국회의원 선거일")

        if self._year == 2020:
            self._add_holiday_apr_15(name)
        elif self._year >= 2007 and (self._year - 2008) % 4 == 0:
            self._add_holiday_2nd_wed_of_apr(name)

        if self._year >= 2007:
            # Presidential Election Day.
            name = tr("대통령 선거일")
            if self._year <= 2024 and (self._year - 2007) % 5 == 0:
                if self._year <= 2012:
                    self._add_holiday_3rd_wed_of_dec(name)
                elif self._year >= 2022:
                    # Moved as per Paragraph 2 of Article 34 due to conflict with
                    # Independence Movement Day (MAR, 1).
                    self._add_holiday_2nd_wed_of_mar(name)
            elif self._year >= 2030 and (self._year - 2030) % 5 == 0:
                self._add_holiday_1st_wed_of_apr(name)

        if self._year >= 2007 and (self._year - 2010) % 4 == 0:
            # Local Election Day.
            name = tr("지방선거일")

            if self._is_tuesday(jun_6) or self._is_wednesday(jun_6) or self._is_thursday(jun_6):
                # Moved as per Paragraph 2 of Article 34 due to conflict with
                # Memorial Day (JUN, 6).
                self._add_holiday_2nd_wed_of_jun(name)
            else:
                self._add_holiday_1st_wed_of_jun(name)

        if self.observed:
            self._populate_observed(dts_observed, three_days_holidays)

    def _populate_bank_holidays(self):
        # Workers' Day.
        name = tr("근로자의날")
        if self._year >= 1994:
            self._add_labor_day(name)
        else:
            self._add_holiday_mar_10(name)


class Korea(SouthKorea):
    def __init__(self, *args, **kwargs) -> None:
        warnings.warn(
            "Korea is deprecated, use SouthKorea instead.", DeprecationWarning, stacklevel=3
        )

        super().__init__(*args, **kwargs)


class KR(SouthKorea):
    pass


class KOR(SouthKorea):
    pass


class SouthKoreaStaticHolidays:
    """South Korea special holidays.

    References:
        * <https://namu.wiki/w/임시공휴일> *
        * <https://namu.wiki/w/공휴일/대한민국> **
        * <https://namu.wiki/w/대체%20휴일%20제도>

    1. Election Dates featured here are the ones prior to the proper recodification to
        Article 34 of the Public Official Election Act (September 2006).
    2. Sabang Day (사방의 날) was technically in the Public Holidays Act itself, but since it was
        only celebrated in 1960, this is being put here.
    """

    # Common Special Holiday Types.

    # National Assembly Election Day.
    national_assembly_election_day = tr("국회의원 선거일")

    # Presidential Election Day.
    presidential_election_day = tr("대통령 선거일")

    # Local Election Day.
    local_election_day = tr("지방선거일")

    # Temporary Public Holiday.
    temporary_public_holiday = tr("임시공휴일")

    # Presidential Inauguration Day.
    presidential_inauguration_day = tr("대통령 취임식")

    # National Conference for Unification Election Day.
    national_conference_for_unification_election_day = tr("통일주체국민회의 선거일")

    # Yushin Constitution Referendum Day.
    yushin_constitution_referendum_day = tr("유신헌법 국민투표일")

    # May 16 Military Coup d'Etat Anniversary.
    may_16_coup_anniversary = tr("5.16 군사혁명 기념일")

    # April 19 Revolution Anniversary.
    apr_19_revolution_anniversary = tr("4.19 혁명 기념일")

    # President Syngman Rhee's Birthday.
    syngman_rhee_birthday = tr("이승만 대통령 탄신일")

    # Armed Forces Day.
    armed_forces_day = tr("국군의 날")

    special_public_holidays = {
        1948: (
            # 1st National Assembly Election.
            (MAY, 10, national_assembly_election_day),
            # 1st Presidential Election.
            (JUL, 20, presidential_election_day),
            # Republic of Korea's United Nations Recognition Celebrations.
            (DEC, 15, tr("국제연합의 대한민국 정부 승인 경축 국민대회")),
        ),
        1949: (
            # Anniversary of the 1st National Assembly Election.
            (MAY, 10, tr("5.10 제헌의회선거 1주년 기념일")),
            # Baekbeom Kim Ku's Funeral Ceremony.
            (JUL, 5, tr("백범 김구 선생 국민장 영결식")),
        ),
        1950: (
            # 2nd National Assembly Election.
            (MAY, 30, national_assembly_election_day),
            # Joint Memorial Service for Fallen Soldiers.
            (JUN, 21, tr("전몰군인 합동위령제")),
        ),
        # Vice Presidential Election.
        1951: (MAY, 16, tr("부통령 선거일")),
        1952: (
            # City/Town/Township-level Local Elections.
            (APR, 25, local_election_day),
            # Provincial-level Local Elections.
            (MAY, 10, local_election_day),
            # 2nd Presidential Election/3rd Vice President Election.
            (AUG, 5, presidential_election_day),
        ),
        # 3rd National Assembly Election.
        1954: (MAY, 20, national_assembly_election_day),
        1956: (
            # 3rd Presidential Election/4th Vice President Election.
            (MAY, 15, presidential_election_day),
            # City/Town/Township-level Local Elections.
            (AUG, 8, local_election_day),
            # Provincial-level Local Elections.
            (AUG, 13, local_election_day),
        ),
        # President Syngman Rhee's Birthday.
        1957: (MAR, 26, syngman_rhee_birthday),
        1958: (
            # 4th National Assembly Election.
            (MAY, 2, national_assembly_election_day),
            # President Syngman Rhee's Birthday.
            (MAR, 26, syngman_rhee_birthday),
        ),
        # President Syngman Rhee's Birthday.
        1959: (MAR, 26, syngman_rhee_birthday),
        1960: (
            # Sabang Day.
            (MAR, 16, tr("사방의 날")),
            # President Syngman Rhee's Birthday.
            (MAR, 26, syngman_rhee_birthday),
            # 4th Presidential Election/5th Vice President Election.
            (MAR, 15, presidential_election_day),
            # 5th National Assembly Election.
            (JUL, 29, national_assembly_election_day),
            # 4th Presidential Election.
            (AUG, 12, presidential_election_day),
            # New Government Celebration Day.
            (OCT, 1, tr("신정부 경축의 날")),
            # Special City/Provincial-level Councillors Local Elections.
            (DEC, 12, local_election_day),
            # City/Town/Township-level Councillors Local Elections.
            (DEC, 19, local_election_day),
            # Special City/Provincial-level Mayors Local Elections.
            (DEC, 26, local_election_day),
            # City/Town/Township-level Governors Local Elections.
            (DEC, 29, local_election_day),
        ),
        # April 19 Revolution Anniversary.
        1961: (APR, 19, apr_19_revolution_anniversary),
        1962: (
            # April 19 Revolution Anniversary.
            (APR, 19, apr_19_revolution_anniversary),
            # May 16 Military Coup d'État Anniversary.
            (MAY, 16, may_16_coup_anniversary),
        ),
        1963: (
            # April 19 Revolution Anniversary.
            (APR, 19, apr_19_revolution_anniversary),
            # May 16 Military Coup d'État Anniversary.
            (MAY, 16, may_16_coup_anniversary),
            # 5th Presidential Election.
            (OCT, 15, presidential_election_day),
            # 6th National Assembly Election.
            (NOV, 26, national_assembly_election_day),
            # President Park Chung Hee's Inauguration Day.
            (DEC, 17, presidential_inauguration_day),
        ),
        # Armed Forces Day.
        1966: (OCT, 1, armed_forces_day),
        1967: (
            # In-lieu observance for New Year's Day.
            (JAN, 4, temporary_public_holiday),
            # 6th Presidential Election.
            (MAY, 3, presidential_election_day),
            # 7th National Assembly Election.
            (JUN, 8, national_assembly_election_day),
            # President Park Chung Hee's Inauguration Day.
            (JUL, 1, presidential_inauguration_day),
        ),
        1969: (
            # Commemoration of the Apollo 11 Moon Landing.
            (JUL, 21, tr("아폴로 11호 달 착륙 기념")),
            # Third-term Constitutional Referendum Day.
            (OCT, 17, tr("삼선 헌법 개정 국민투표일")),
        ),
        1971: (
            # 7th Presidential Election.
            (APR, 27, presidential_election_day),
            # 8th National Assembly Election.
            (MAY, 25, national_assembly_election_day),
            # President Park Chung Hee's Inauguration Day.
            (JUL, 1, presidential_inauguration_day),
        ),
        1972: (
            # Yushin Constitution Referendum Day.
            (NOV, 21, yushin_constitution_referendum_day),
            # 1st National Conference for Unification Election Day.
            (DEC, 15, national_conference_for_unification_election_day),
            # 8th Presidential Election.
            (DEC, 23, presidential_election_day),
            # President Park Chung Hee's Inauguration Day.
            (DEC, 27, presidential_inauguration_day),
        ),
        # 9th National Assembly Election.
        1973: (FEB, 27, national_assembly_election_day),
        # First Lady Yuk Young-soo's Funeral Ceremony.
        1974: (AUG, 19, tr("대통령 영부인 육영수 여사 국민장 영결식")),
        # Yushin Constitution Referendum Day.
        1975: (FEB, 12, yushin_constitution_referendum_day),
        1978: (
            # 2nd National Conference for Unification Election Day.
            (MAY, 18, national_conference_for_unification_election_day),
            # 9th Presidential Election.
            (JUL, 6, presidential_election_day),
            # 10th National Assembly Election.
            (DEC, 12, national_assembly_election_day),
            # President Park Chung Hee's Inauguration Day.
            (DEC, 27, presidential_inauguration_day),
        ),
        1979: (
            # President Park Chung Hee's Funeral Ceremony.
            (NOV, 3, tr("박정희 대통령 국장 영결식")),
            # 10th Presidential Election.
            (DEC, 6, presidential_election_day),
            # President Choi Kyu-hah's Inauguration Day.
            (DEC, 21, presidential_inauguration_day),
        ),
        1980: (
            # 11th Presidential Election.
            (AUG, 27, presidential_election_day),
            # President Chun Doo-hwan's Inauguration Day.
            (SEP, 1, presidential_inauguration_day),
            # 5th Republic Constitutional Referendum Day.
            (OCT, 22, tr("제5공화국 헌법 개정 국민투표일")),
        ),
        1981: (
            # Electoral College Election Day.
            (FEB, 11, tr("선거를 위한 선거인단 선일")),
            # 12th Presidential Election.
            (FEB, 25, presidential_election_day),
            # President Chun Doo-hwan's Inauguration Day.
            (MAR, 3, presidential_inauguration_day),
            # 11th National Assembly Election.
            (MAR, 25, national_assembly_election_day),
        ),
        # Added due to overlaps between Chuseok and Armed Forces Day.
        1982: (OCT, 2, temporary_public_holiday),
        # 12th National Assembly Election.
        1985: (FEB, 12, national_assembly_election_day),
        1987: (
            # 13th Presidential Election.
            (DEC, 16, presidential_election_day),
            # 6th Republic Constitutional Referendum Day.
            (OCT, 27, tr("제6공화국 헌법 개정 국민투표일")),
        ),
        1988: (
            # President Roh Tae-woo's Inauguration Day.
            (FEB, 25, presidential_inauguration_day),
            # 13th National Assembly Election.
            (APR, 26, national_assembly_election_day),
            # 1988 Seoul Olympics Opening Ceremony.
            (SEP, 17, tr("1988 서울 올림픽 개막식")),
        ),
        1991: (
            # District/City/County-level Local Elections.
            (MAR, 26, local_election_day),
            # Metropolitan/Provincial-level Local Elections.
            (JUN, 20, local_election_day),
        ),
        1992: (
            # 14th National Assembly Election.
            (MAR, 24, national_assembly_election_day),
            # 14th Presidential Election.
            (DEC, 18, presidential_election_day),
        ),
        # 1st Nationwide Local Election.
        1995: (JUN, 27, local_election_day),
        # 15th National Assembly Election.
        1996: (APR, 11, national_assembly_election_day),
        # 15th Presidential Election.
        1997: (DEC, 18, presidential_election_day),
        # 2nd Nationwide Local Election.
        1998: (JUN, 4, local_election_day),
        # 16th National Assembly Election.
        2000: (APR, 13, national_assembly_election_day),
        2002: (
            # 3rd Nationwide Local Election.
            (JUN, 13, local_election_day),
            # 2002 FIFA World Cup National Team Semi-Finals Celebrations.
            (JUL, 1, tr("2002년 한일 월드컵 대표팀 4강 진출")),
            # 16th Presidential Election.
            (DEC, 19, presidential_election_day),
        ),
        # 17th National Assembly Election.
        2004: (APR, 15, national_assembly_election_day),
        # 4th Nationwide Local Election.
        2006: (MAY, 31, local_election_day),
        # Added to help cope with MERS Pandemic fatigue.
        2015: (AUG, 14, temporary_public_holiday),
        # Added to fill in holiday gaps between Children's Day and Saturday.
        2016: (MAY, 6, temporary_public_holiday),
        2017: (
            # Special Presidential Election due to Park Geun-hye's impeachment.
            (MAY, 9, presidential_election_day),
            # Added to create a 10-day long holiday period.
            (OCT, 2, temporary_public_holiday),
        ),
        # Added to help cope with Covid-19 Pandemic fatigue.
        2020: (AUG, 17, temporary_public_holiday),
        # Added to create a 6-day long holiday period.
        2023: (OCT, 2, temporary_public_holiday),
        # 76th Anniversary of the Armed Forces of Korea.
        2024: (OCT, 1, armed_forces_day),
        2025: (
            # Added to create a 6-day long holiday period.
            (JAN, 27, temporary_public_holiday),
            # Special Presidential Election (21st) due to Yoon Seok-yeol's impeachment.
            (JUN, 3, presidential_election_day),
        ),
    }
    # Pre-2014 Alternate Holidays
    # https://namu.wiki/w/대체%20휴일%20제도#s-4.2.1
    special_public_holidays_observed = {
        1959: (APR, 6, tr("식목일")),
        1960: (
            (JUL, 18, tr("제헌절")),
            (OCT, 10, tr("한글날")),
            (DEC, 26, tr("기독탄신일")),
        ),
        1989: (OCT, 2, armed_forces_day),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.groups.balinese_saka import BalineseSakaCalendarHolidays
from holidays.groups.buddhist import BuddhistCalendarHolidays
from holidays.groups.burmese import BurmeseCalendarHolidays
from holidays.groups.chinese import ChineseCalendarHolidays
from holidays.groups.christian import ChristianHolidays
from holidays.groups.custom import StaticHolidays
from holidays.groups.eastern import EasternCalendarHolidays
from holidays.groups.hebrew import HebrewCalendarHolidays
from holidays.groups.hindu import HinduCalendarHolidays
from holidays.groups.international import InternationalHolidays
from holidays.groups.islamic import IslamicHolidays
from holidays.groups.mandaean import MandaeanHolidays
from holidays.groups.mongolian import MongolianCalendarHolidays
from holidays.groups.persian import PersianCalendarHolidays
from holidays.groups.sinhala import SinhalaCalendarHolidays
from holidays.groups.thai import ThaiCalendarHolidays
from holidays.groups.tibetan import TibetanCalendarHolidays



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SUN_TO_NEXT_MON,
    SUN_TO_NEXT_TUE,
    SAT_SUN_TO_NEXT_MON,
)


class Jamaica(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Jamaica holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Jamaica>
        * <https://web.archive.org/web/20241110125301/https://www.mlss.gov.jm/wp-content/uploads/2017/11/The-Holidays-Public-General-Act.pdf>
    """

    country = "JM"
    observed_label = "%s (observed)"

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day
        self._add_observed(self._add_new_years_day("New Year's Day"))

        # Ash Wednesday
        self._add_ash_wednesday("Ash Wednesday")

        # Good Friday
        self._add_good_friday("Good Friday")

        # Easter Monday
        self._add_easter_monday("Easter Monday")

        # National Labour Day
        self._add_observed(
            self._add_holiday_may_23("National Labour Day"), rule=SAT_SUN_TO_NEXT_MON
        )

        # Emancipation Day
        if self._year >= 1998:
            self._add_observed(self._add_holiday_aug_1("Emancipation Day"))

        # Independence Day
        self._add_observed(self._add_holiday_aug_6("Independence Day"))

        # National Heroes Day
        self._add_holiday_3rd_mon_of_oct("National Heroes Day")

        # Christmas Day
        self._add_observed(self._add_christmas_day("Christmas Day"), rule=SUN_TO_NEXT_TUE)

        # Boxing Day
        self._add_observed(self._add_christmas_day_two("Boxing Day"))


class JM(Jamaica):
    pass


class JAM(Jamaica):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import APR, MAY, JUN, JUL, AUG, SEP, OCT, DEC
from holidays.groups import (
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON


class Senegal(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays, StaticHolidays
):
    """Senegal holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Senegal>
        * [Law 63-51](https://web.archive.org/web/20250604134724/https://www.dri.gouv.sn/sites/default/files/LOI/1963/63_51.pdf)
        * [Law 74-52](https://web.archive.org/web/20250608105243/https://www.dri.gouv.sn/sites/default/files/an-documents/LOI%20N1974%2052%20DU%204%20NOVEMBRE%201974.pdf)
        * [Law 83-54](https://web.archive.org/web/20250608105419/https://www.dri.gouv.sn/sites/default/files/LOI/1983/comP4%20loi%20decentralisation%20et%20travail/LOI%20N%20198354%20DU%2018%20FEVRIER%201983/LOI%20N%20198354%20DU%2018%20FEVRIER%201983.pdf)
        * [Law 2013-06](https://web.archive.org/web/20250604215148/https://natlex.ilo.org/dyn/natlex2/natlex2/files/download/97261/SEN-97261.pdf)
    """

    country = "SN"
    default_language = "fr_SN"
    # %s (estimated).
    estimated_label = tr("%s (estimé)")
    # %s (observed).
    observed_label = tr("%s (observé)")
    # %s (observed, estimated).
    observed_estimated_label = tr("%s (observé, estimé)")
    # First official source with holidays dates (Law 63-51).
    start_year = 1964
    supported_languages = ("en_US", "fr_SN")

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=SenegalIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, SenegalStaticHolidays)
        # Law 74-52.
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        kwargs.setdefault("observed_since", 1975)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        if self._year >= 1983:
            # Ashura.
            self._add_ashura_day(tr("Tamxarit"))

        if self._year >= 2014:
            # Grand Magal of Touba.
            self._add_grand_magal_of_touba(tr("Grand Magal de Touba"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("Journée du Maouloud"))

        # Eid al-Fitr.
        for dt in self._add_eid_al_fitr_day(tr("Journée de la Korité")):
            self._add_observed(dt)

        # Eid al-Adha.
        for dt in self._add_eid_al_adha_day(tr("Journée de la Tabaski")):
            self._add_observed(dt)

        # New Year's Day.
        self._add_new_years_day(tr("Jour de l'an"))

        if 1983 <= self._year <= 1989:
            # Senegambia Confederation Day.
            self._add_holiday_feb_1(tr("Fête de la Confédération de la Sénégambie"))

        # Independence Day.
        name = tr("Fête de l'Indépendance")
        if self._year >= 1975:
            self._add_holiday_apr_4(name)
        else:
            self._add_holiday_jul_14(name)

        # Easter Monday.
        self._add_easter_monday(tr("Lundi de Pâques"))

        # Labor Day.
        self._add_labor_day(tr("Fête du Travail"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Jeudi de l'Ascension"))

        # Whit Monday.
        self._add_whit_monday(tr("Lundi de Pentecôte"))

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Assomption"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Toussaint"))

        # Christmas Day.
        self._add_christmas_day(tr("Noël"))


class SN(Senegal):
    pass


class SEN(Senegal):
    pass


class SenegalIslamicHolidays(_CustomIslamicHolidays):
    # https://web.archive.org/web/20250609044648/https://www.timeanddate.com/holidays/senegal/tamkharit
    ASHURA_DATES_CONFIRMED_YEARS = (2020, 2024)
    ASHURA_DATES = {
        2024: (JUL, 17),
    }

    # https://web.archive.org/web/20250609061446/https://www.timeanddate.com/holidays/senegal/grand-magal-de-touba
    GRAND_MAGAL_OF_TOUBA_DATES_CONFIRMED_YEARS = (2020, 2024)
    GRAND_MAGAL_OF_TOUBA_DATES = {
        2020: (OCT, 6),
        2021: (SEP, 26),
        2022: (SEP, 15),
        2023: (SEP, 4),
        2024: (AUG, 23),
    }

    # https://web.archive.org/web/20250609091559/https://www.timeanddate.com/holidays/senegal/maouloud
    MAWLID_DATES_CONFIRMED_YEARS = (2020, 2024)

    # https://web.archive.org/web/20250609092813/https://www.timeanddate.com/holidays/senegal/korite
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2020, 2025)
    EID_AL_FITR_DATES = {
        2021: (MAY, 12),
        2022: (MAY, 1),
        2023: (APR, 22),
    }

    # https://web.archive.org/web/20250609102658/https://www.timeanddate.com/holidays/senegal/tabaski
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2020, 2024)
    EID_AL_ADHA_DATES = {
        2021: (JUL, 21),
        2022: (JUL, 10),
        2023: (JUN, 29),
        2024: (JUN, 17),
    }


class SenegalStaticHolidays:
    """Senegal special holidays.

    References:
        * [29th October, 2018 Public holiday](https://web.archive.org/web/20250608105852/https://www.juriafrica.com/lex/decret-2018-1942-26-octobre-2018-48947.htm)
        * [26th December, 2022 Public holiday](https://web.archive.org/web/20250608135605/https://primature.sn/publications/conseil-des-ministres/conseil-des-ministres-du-22-decembre-2022)
    """

    # Public holiday.
    name = tr("Jour férié")
    special_public_holidays = {
        2018: (OCT, 29, name),
        2022: (DEC, 26, name),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import APR, MAY, JUN, JUL, SEP, DEC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    MON_TO_NEXT_TUE,
    SAT_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class Montserrat(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Montserrat holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Montserrat>
        * [Public holidays Act, 2017](https://web.archive.org/web/20240619074948/http://agc.gov.ms/wp-content/uploads/2011/10/Act-No.-3-of-2017-Public-Holidays-Amendment-Act-20171.pdf)
        * [Public holidays Act, 2019](https://web.archive.org/web/20241025070627/https://www.gov.ms/wp-content/uploads/2020/06/Public-Holidays-Act.pdf)
        * [National Day of Prayer and Thanksgiving, 2021](https://web.archive.org/web/20220809144840/https://www.gov.ms/2021/07/16/wednesday-july-21-2021-public-holiday-national-day-of-prayer-thanksgiving/)
        * [National Day of Prayer and Thanksgiving, 2022](https://web.archive.org/web/20220816124619/https://www.gov.ms/2022/07/19/public-holiday-for-national-day-of-prayer-and-thanksgiving-to-be-held-on-july-20-2022/)
        * [National Day of Prayer and Thanksgiving](https://web.archive.org/web/20240711221313/https://www.gov.ms/wp-content/uploads/2023/05/SRO-No.-15-of-2023-Public-Holidays-Amendment-of-Schedule-Order.pdf)
        * [Queen's Platinum Jubilee, 2022](https://web.archive.org/web/20250711160439/https://www.gov.ms/tag/public-holidays/)
        * [Montserrat Public holidays, 2022](https://web.archive.org/web/20220809030551/https://www.gov.ms/wp-content/uploads/2021/12/Public-Holidays-2022-1.jpeg)
        * [Montserrat Public holidays, 2023](https://web.archive.org/web/20241126232715/https://www.gov.ms/wp-content/uploads/2023/02/Public-Holidays-Montserrat-2023_page-0001.jpg)
        * [Montserrat Public holidays, 2024](https://web.archive.org/web/20240421112540/https://www.gov.ms/wp-content/uploads/2023/12/Public-Holidays-Montserrat-2024.docx.pdf)
        * [King's Birthday, 2024](https://web.archive.org/web/20250711202228/https://parliament.ms/wp-content/uploads/2024/09/SRO-No.-18-of-2024-PROCOLAMATION-APPOINTING-MONDAY-17-JUNE-2024-AS-A-PUBLIC-HOLIDAY.pdf)
        * [Montserrat Public holidays, 2025](https://web.archive.org/web/20250711160324/https://www.gov.ms/wp-content/uploads/2025/01/2025-Public-Holidays-on-Montserrat.pdf)
    """

    country = "MS"
    default_language = "en_MS"
    # %s (observed).
    observed_label = "%s (observed)"
    # Public holidays Act, 2017.
    start_year = 2017
    supported_languages = ("en_MS", "en_US")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, MontserratStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        self._add_observed(
            # New Year's Day.
            self._add_new_years_day(tr("New Year's Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE + MON_TO_NEXT_TUE,
        )

        # Saint Patrick's Day.
        self._add_observed(self._add_saint_patricks_day(tr("Saint Patrick's Day")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # Labor Day.
        self._add_holiday_1st_mon_of_may(tr("Labour Day"))

        name = (
            # King's Birthday.
            tr("King's Birthday")
            if self._year >= 2023
            # Queen's Birthday.
            else tr("Queen's Birthday")
        )
        sovereign_birthday_dts = {
            2022: (JUN, 2),
            2023: (JUN, 19),
            2024: (JUN, 17),
        }
        sovereign_birthday_dt = (
            self._add_holiday(name, dt)
            if (dt := sovereign_birthday_dts.get(self._year))
            else self._add_holiday_2_days_past_2nd_sat_of_jun(name)
        )

        # Whit Monday.
        name = tr("Whit Monday")
        whit_monday_dt = self._add_whit_monday(name)
        if whit_monday_dt == sovereign_birthday_dt:
            self._add_observed(whit_monday_dt, name=name, rule=MON_TO_NEXT_TUE)

        if self._year >= 2021:
            day_of_prayer_dts = {
                2021: (JUL, 21),
                2022: (JUL, 20),
            }
            name = (
                # National Day of Prayer and Thanksgiving.
                tr("National Day of Prayer and Thanksgiving")
                if self._year <= 2023
                # Day of Prayer and Thanksgiving.
                else tr("Day of Prayer and Thanksgiving")
            )
            if dt := day_of_prayer_dts.get(self._year):
                self._add_holiday(name, dt)
            else:
                self._add_holiday_2nd_wed_of_jul(name)

        # Emancipation Day.
        self._add_holiday_1st_mon_of_aug(tr("Emancipation Day"))

        self._add_observed(
            # Christmas Day.
            self._add_christmas_day(tr("Christmas Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        self._add_observed(
            # Boxing Day.
            self._add_christmas_day_two(tr("Boxing Day")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

        # Festival Day.
        name = tr("Festival Day")
        self._add_new_years_eve(name)
        self._add_observed((self._year - 1, DEC, 31), name, rule=SAT_TO_NEXT_MON)


class MS(Montserrat):
    pass


class MSR(Montserrat):
    pass


class MontserratStaticHolidays(StaticHolidays):
    """Montserrat special holidays.

    References:
        * [September 14th, 2018](https://web.archive.org/web/20220810224300/https://www.gov.ms/wp-content/uploads/2020/11/SRO.-No.-35-of-2018-Proclamation-Declaring-Friday-14-September-2018-as-a-public-holiday.pdf)
        * [May 10th, 2019](https://web.archive.org/web/20240626113529/https://parliament.ms/wp-content/uploads/2022/02/SRO-No-13-of-2019-Proclamation-Declaring-Friday-10-May-2019-as-a-publi._.pdf)
        * [July 15th, 2020](https://web.archive.org/web/20220810225456/https://www.gov.ms/wp-content/uploads/2020/08/SRO.-No.-40-of-2020-Proclamation-Declaring-Wednesday-15-July-2020-as-a-Public-Holiday.pdf)
        * [National Day of Mourning](https://web.archive.org/web/20240617072858/https://www.parliament.ms/wp-content/uploads/2022/09/SRO-No.43-of-2022-Proclamation-Appointing-Monday-19-September-2022-a-Public-Holiday.pdf)
        * [Coronation of King Charles III](https://web.archive.org/web/20241126232715/https://www.gov.ms/wp-content/uploads/2023/02/Public-Holidays-Montserrat-2023_page-0001.jpg)
    """

    # Special Public Holiday.
    name = tr("Special Public Holiday")

    special_public_holidays = {
        2018: (SEP, 14, name),
        2019: (MAY, 10, name),
        2020: (JUL, 15, name),
        2022: (
            # Platinum Jubilee of Elizabeth II.
            (JUN, 3, tr("Platinum Jubilee of Elizabeth II")),
            # National Day of Mourning.
            (SEP, 19, tr("National Day of Mourning")),
        ),
        2023: (
            (APR, 12, name),
            # Coronation of King Charles III.
            (MAY, 6, tr("Coronation of King Charles III")),
        ),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import GREGORIAN_CALENDAR
from holidays.calendars.julian import JULIAN_CALENDAR
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Moldova(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Moldova holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Moldova>
        * <https://archive.org/details/httpswww.legis.mdcautaregetresultsdoc_id133686langro>
    """

    country = "MD"
    default_language = "ro"
    supported_languages = ("en_US", "ro", "uk")
    start_year = 1991

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self, JULIAN_CALENDAR)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Anul Nou"))

        name = (
            # Christmas Day (by old style).
            tr("Nașterea lui Iisus Hristos (Crăciunul pe stil vechi)")
            if self._year >= 2014
            # Christmas Day.
            else tr("Nașterea lui Iisus Hristos (Crăciunul)")
        )
        self._add_christmas_day(name)
        self._add_christmas_day_two(name)

        # International Women's Day.
        self._add_womens_day(tr("Ziua internatională a femeii"))

        # Easter.
        name = tr("Paștele")
        self._add_easter_sunday(name)
        self._add_easter_monday(name)

        # Day of Rejoicing.
        self._add_holiday_8_days_past_easter(tr("Paștele blajinilor"))

        # International Workers' Solidarity Day.
        self._add_labor_day(tr("Ziua internaţională a solidarităţii oamenilor muncii"))

        self._add_world_war_two_victory_day(
            # Victory Day and Commemoration of the heroes fallen for
            # Independence of Fatherland.
            tr("Ziua Victoriei și a comemorării eroilor căzuţi pentru Independenţa Patriei"),
            is_western=False,
        )

        if self._year >= 2017:
            # Europe Day.
            self._add_europe_day(tr("Ziua Europei"))

        if self._year >= 2016:
            # International Children's Day.
            self._add_childrens_day(tr("Ziua Ocrotirii Copilului"))

        # Republic of Moldova Independence Day.
        self._add_holiday_aug_27(tr("Ziua independenţei Republicii Moldova"))

        # National Language Day.
        self._add_holiday_aug_31(tr("Limba noastră"))

        if self._year >= 2013:
            self._add_christmas_day(
                # Christmas Day (by new style).
                tr("Nașterea lui Iisus Hristos (Crăciunul pe stil nou)"),
                GREGORIAN_CALENDAR,
            )


class MD(Moldova):
    pass


class MDA(Moldova):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.groups import ChristianHolidays, InternationalHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class Uganda(HolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Uganda holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Uganda>
        * [Public Holidays Act, Cap. 255](https://web.archive.org/web/20250621140041/https://ulii.org/akn/ug/act/1965/23/eng@2000-12-31)
        * [National Events and Calendar](https://web.archive.org/web/20250618053410/https://www.vicepresident.go.ug/national-events-and-calender/)
        * [National Heroes Day](https://web.archive.org/web/20230622162351/https://publicholidays.africa/uganda/national-heroes-day/)
        * [NRM Liberation Day](https://web.archive.org/web/20240124064853/https://thenilewires.com/38th-nrm-a-liberation-day-celebrations-minister-outlines-nrm-achievements/)
        * [Archbishop Janani Luwum Day](https://web.archive.org/web/20200805133919/https://www.newvision.co.ug/news/1417242/uganda-declared-february-archbishop-janani-luwum-public-holiday)
        * [School Calendar 2024](https://web.archive.org/web/20240519021615/https://www.education.go.ug/wp-content/uploads/2023/11/SCHOOL-CALENDAR.pdf)
        * [Public Holidays 2025](https://web.archive.org/web/20250710193157/https://publicholidays.co.ug/)
    """

    country = "UG"
    # %s (estimated).
    estimated_label = "%s (estimated)"
    # Uganda gained independence from the United Kingdom on October 9, 1962.
    start_year = 1963

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(self, show_estimated=islamic_show_estimated)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day("New Year's Day")

        if self._year >= 1987:
            # NRM Liberation Day.
            self._add_holiday_jan_26("NRM Liberation Day")

        if self._year >= 2016:
            # Archbishop Janani Luwum Day.
            self._add_holiday_feb_16("Archbishop Janani Luwum Day")

        # Women's Day.
        self._add_womens_day("Women's Day")

        # Good Friday.
        self._add_good_friday("Good Friday")

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        # Labor Day.
        self._add_labor_day("Labour Day")

        # Uganda Martyrs' Day.
        self._add_holiday_jun_3("Uganda Martyrs' Day")

        if self._year >= 2001:
            # National Heroes' Day.
            self._add_holiday_jun_9("National Heroes' Day")

        # Independence Day.
        self._add_holiday_oct_9("Independence Day")

        # Christmas Day.
        self._add_christmas_day("Christmas Day")

        # Boxing Day.
        self._add_christmas_day_two("Boxing Day")

        # Eid al-Fitr.
        self._add_eid_al_fitr_day("Eid al-Fitr")

        # Eid al-Adha.
        self._add_eid_al_adha_day("Eid al-Adha")


class UG(Uganda):
    pass


class UGA(Uganda):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomIslamicHolidays
from holidays.calendars.gregorian import FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, FRI, SAT
from holidays.groups import ChristianHolidays, InternationalHolidays, IslamicHolidays
from holidays.holiday_base import HolidayBase


class Bangladesh(HolidayBase, ChristianHolidays, InternationalHolidays, IslamicHolidays):
    """Bangladesh holidays.

    References:
        * <https://web.archive.org/web/20241109215908/https://mopa.gov.bd/sites/default/files/files/mopa.gov.bd/public_holiday/61c35b73_e335_462a_9bcf_4695b23b6d82/reg4-2019-212.PDF>
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Bangladesh>
        * <https://web.archive.org/web/20260427032339/https://cgbdsydney.gov.bd/about-us/holidays/>
        * <https://web.archive.org/web/20251118113843/https://publicholidays.com.bd/>
    """

    country = "BD"
    default_language = "bn"
    # %s (estimated).
    estimated_label = tr("%s (আনুমানিক)")

    supported_languages = ("ar", "bn", "en_BD", "en_US")
    weekend = {FRI, SAT}

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=BangladeshIslamicHolidays, show_estimated=islamic_show_estimated
        )
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Martyrs' Day and International Mother Language Day.
        self._add_holiday_feb_21(tr("শহীদ দিবস ও আন্তর্জাতিক মাতৃভাষা দিবস"))

        if self._year <= 2024:
            # Sheikh Mujibur Rahman's Birthday.
            self._add_holiday_mar_17(tr("জাতির পিতা বঙ্গবন্ধু শেখ মুজিবুর রহমান এর জন্মদিবস"))

        # Independence Day.
        self._add_holiday_mar_26(tr("স্বাধীনতা দিবস"))

        # Bengali New Year's Day.
        self._add_holiday_apr_14(tr("পহেলা বৈশাখ"))

        # May Day.
        self._add_labor_day(tr("মে দিবস"))

        if self._year >= 2025:
            # July Mass Uprising Day.
            self._add_holiday_aug_5(tr("জুলাই গণ-অভ্যুত্থান দিবস"))

        if self._year <= 2024:
            # National Mourning Day.
            self._add_holiday_aug_15(tr("জাতীয় শোক দিবস"))

        # Victory Day.
        self._add_holiday_dec_16(tr("বিজয় দিবস"))

        # Christmas Day.
        self._add_christmas_day(tr("বড়দিন"))

        # Ashura.
        self._add_ashura_day(tr("আশুরা"))

        # Prophet's Birthday.
        self._add_mawlid_day(tr("ঈদে মিলাদুন্নবী"))

        # Mid-Sha'ban.
        self._add_mid_shaban_day(tr("শবে বরাত"))

        # Laylat al-Qadr.
        self._add_laylat_al_qadr_day(tr("শবে কদর"))

        # Jumu'atul-Wida.
        self._add_jumuatul_wida(tr("জুমাতুল বিদা"))

        # Eid al-Fitr.
        name = tr("ঈদুল ফিতর")
        self._add_eid_al_fitr_day(name)
        self._add_eid_al_fitr_day_two(name)
        self._add_eid_al_fitr_day_three(name)

        # Eid al-Adha.
        name = tr("ঈদুল আজহা")
        self._add_eid_al_adha_day(name)
        self._add_eid_al_adha_day_two(name)
        self._add_eid_al_adha_day_three(name)


class BD(Bangladesh):
    pass


class BGD(Bangladesh):
    pass


class BangladeshIslamicHolidays(_CustomIslamicHolidays):
    # https://www.timeanddate.com/holidays/bangladesh/ashura
    ASHURA_DATES_CONFIRMED_YEARS = (2022, 2025)
    ASHURA_DATES = {
        2022: (AUG, 9),
        2023: (JUL, 29),
        2024: (JUL, 17),
        2025: (JUL, 6),
    }

    # https://www.timeanddate.com/holidays/bangladesh/eid-ul-adha-holiday-1
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2022, 2025)
    EID_AL_ADHA_DATES = {
        2022: (JUL, 10),
        2023: (JUN, 29),
        2024: (JUN, 17),
        2025: (JUN, 7),
    }

    # https://www.timeanddate.com/holidays/bangladesh/eid-ul-fitr
    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2022, 2026)
    EID_AL_FITR_DATES = {
        2022: (MAY, 3),
        2023: (APR, 22),
        2024: (APR, 10),
        2025: (MAR, 31),
        2026: (MAR, 21),
    }

    # https://www.timeanddate.com/holidays/bangladesh/jumatul-bidah
    JUMUATUL_WIDA_DATES_CONFIRMED_YEARS = (2022, 2026)
    JUMUATUL_WIDA_DATES = {
        2022: (APR, 29),
        2023: (APR, 21),
        2024: (APR, 5),
        2025: (MAR, 28),
        2026: (MAR, 20),
    }

    # https://www.timeanddate.com/holidays/bangladesh/shab-e-qadr
    LAYLAT_AL_QADR_DATES_CONFIRMED_YEARS = (2022, 2026)
    LAYLAT_AL_QADR_DATES = {
        2022: (APR, 29),
        2023: (APR, 19),
        2024: (APR, 7),
        2025: (MAR, 28),
        2026: (MAR, 17),
    }

    # https://www.timeanddate.com/holidays/bangladesh/eid-e-milad-un-nabi
    MAWLID_DATES_CONFIRMED_YEARS = (2022, 2025)
    MAWLID_DATES = {
        2022: (OCT, 9),
        2023: (SEP, 28),
        2024: (SEP, 16),
        2025: (SEP, 6),
    }

    # https://www.timeanddate.com/holidays/bangladesh/shab-e-barat
    IMAM_MAHDI_BIRTHDAY_DATES_CONFIRMED_YEARS = (2022, 2026)
    IMAM_MAHDI_BIRTHDAY_DATES = {
        2022: (MAR, 19),
        2023: (MAR, 8),
        2024: (FEB, 26),
        2025: (FEB, 15),
        2026: (FEB, 3),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class Congo(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Congo holidays.

    References:
        * [Loi N° 2-94](https://web.archive.org/web/20241003070938/http://mokili.free.fr/jours_feries.php)
        * [Loi N° 18-2010](https://web.archive.org/web/20250427173832/https://www.finances.gouv.cg/sites/default/files/documents/n¯18-2010%20du%2027%20novembre%202010.PDF)

    Cross-Checked With:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_the_Republic_of_the_Congo>
    """

    country = "CG"
    default_language = "fr"
    supported_languages = ("en_US", "fr")
    # Loi N° 2-94 of 1 March 1994.
    start_year = 1994

    def __init__(self, *args, **kwargs) -> None:
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Jour de l'An"))

        # Easter Monday.
        self._add_easter_monday(tr("Lundi de Pâques"))

        # Labor Day.
        self._add_labor_day(tr("Fête du Travail"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Ascension"))

        # Whit Monday.
        self._add_whit_monday(tr("Lundi de Pentecôte"))

        # Reconciliation Day.
        self._add_holiday_jun_10(tr("Fête de la Réconciliation"))

        # National Day.
        self._add_holiday_aug_15(tr("Fête Nationale"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Toussaint"))

        if self._year >= 2010:
            # Republic Day.
            self._add_holiday_nov_28(tr("Jour de la République"))

        # Christmas Day.
        self._add_christmas_day(tr("Noël"))


class CG(Congo):
    pass


class COG(Congo):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.constants import OPTIONAL, PUBLIC
from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.holiday_base import HolidayBase


class CaboVerde(HolidayBase, ChristianHolidays, InternationalHolidays):
    """Cabo Verde holidays.

    References:
        * [Public holidays in Cape Verde](https://en.wikipedia.org/wiki/Public_holidays_in_Cape_Verde)
        * [Legislação Municipal Cabo-Verdiana](https://web.archive.org/web/20180820172827/http://www.interface.gov.cv/index.php?option=com_docman&task=doc_download&gid=12&Itemid=314)
        * [Feriados Bancários - Banco de Cabo Verde](https://web.archive.org/web/20250629042854/https://www.bcv.cv/pt/SistemadePagamentos/feriados_bancarios/Paginas/FeriadosBancarios.aspx)
        * [Feriados Públicos - Feel Cabo Verde](https://web.archive.org/web/20250419201823/https://feelcaboverde.com/feriados-publicos/)
        * [Public Holidays - Feel Cape Verde](https://web.archive.org/web/20250419202739/https://feelcaboverde.com/en/public-holidays-cape-verde/)
        * [Democracy Day](https://web.archive.org/web/20250420093850/https://cmsv.cv/dia-da-liberdade-e-democracia-de-cabo-verde/)
    """

    country = "CV"
    default_language = "pt_CV"
    start_year = 1976
    subdivisions = (
        "BR",  # Brava.
        "BV",  # Boa Vista.
        "CA",  # Santa Catarina.
        "CF",  # Santa Catarina do Fogo.
        "CR",  # Santa Cruz.
        "MA",  # Maio.
        "MO",  # Mosteiros.
        "PA",  # Paul.
        "PN",  # Porto Novo.
        "PR",  # Praia.
        "RB",  # Ribeira Brava
        "RG",  # Ribeira Grande.
        "RS",  # Ribeira Grande de Santiago.
        "SD",  # São Domingos.
        "SF",  # São Filipe.
        "SL",  # Sal.
        "SM",  # São Miguel.
        "SO",  # São Lourenço dos Órgãos.
        "SS",  # São Salvador do Mundo.
        "SV",  # São Vicente.
        "TA",  # Tarrafal.
        "TS",  # Tarrafal de São Nicolau.
    )

    subdivisions_aliases = {
        "Brava": "BR",
        "Boa Vista": "BV",
        "Santa Catarina": "CA",
        "Santa Catarina do Fogo": "CF",
        "Santa Cruz": "CR",
        "Maio": "MA",
        "Mosteiros": "MO",
        "Paul": "PA",
        "Porto Novo": "PN",
        "Praia": "PR",
        "Ribeira Brava": "RB",
        "Ribeira Grande": "RG",
        "Ribeira Grande de Santiago": "RS",
        "São Domingos": "SD",
        "São Filipe": "SF",
        "Sal": "SL",
        "São Miguel": "SM",
        "São Lourenço dos Órgãos": "SO",
        "São Salvador do Mundo": "SS",
        "São Vicente": "SV",
        "Tarrafal": "TA",
        "Tarrafal de São Nicolau": "TS",
    }
    supported_categories = (OPTIONAL, PUBLIC)
    supported_languages = ("de", "en_US", "es", "fr", "pt_CV")

    def __init__(self, *args, **kwargs) -> None:
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("Ano Novo"))

        # Law # 95/V/99 of March 22nd.
        if self._year >= 2000:
            # Democracy and Freedom Day.
            self._add_holiday_jan_13(tr("Dia da Liberdade e da Democracia"))

        # National Heroes Day.
        self._add_holiday_jan_20(tr("Dia da Nacionalidade e dos Heróis Nacionais"))

        # Ash Wednesday.
        self._add_ash_wednesday(tr("Quarta-feira de Cinzas"))

        # Good Friday.
        self._add_good_friday(tr("Sexta-feira Santa"))

        # Easter Sunday.
        self._add_easter_sunday(tr("Páscoa"))

        # Worker's Day.
        self._add_labor_day(tr("Dia do Trabalhador"))

        # Law # 69/VI/2005 of May 31st.
        if self._year >= 2005:
            # International Children's Day.
            self._add_childrens_day(tr("Dia Mundial da Criança"))

        # Independence Day.
        self._add_holiday_jul_5(tr("Dia da Independência Nacional"))

        # Assumption Day.
        self._add_assumption_of_mary_day(tr("Dia da Assunção"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Dia de Todos os Santos"))

        # Christmas Day.
        self._add_christmas_day(tr("Dia do Natal"))

    def _populate_optional_holidays(self):
        # Holy Thursday.
        self._add_holy_thursday(tr("Quinta-Feira Santa"))

        # Mother's Day.
        self._add_holiday_2nd_sun_of_may(tr("Dia das Mães"))

        # Father's Day.
        self._add_holiday_3rd_sun_of_jun(tr("Dia dos Pais"))

    def _populate_subdiv_br_public_holidays(self):
        # Law # 93/82 of November 6th.
        if self._year >= 1983:
            # Brava Municipality Day.
            self._add_holiday_jun_24(tr("Dia do Município da Brava"))

    def _populate_subdiv_bv_public_holidays(self):
        # Law # 93/82 of November 6th.
        if self._year >= 1983:
            # Boa Vista Municipality Day.
            self._add_holiday_jul_4(tr("Dia do Município da Boa Vista"))

    def _populate_subdiv_ca_public_holidays(self):
        # Law # 93/82 of November 6th.
        if self._year >= 1982:
            # Santa Catarina de Santiago Municipality Day.
            self._add_holiday_nov_25(tr("Dia do Município de Santa Catarina de Santiago"))

    def _populate_subdiv_cf_public_holidays(self):
        # Law # 66/VI/2005 of May 9th.
        if self._year >= 2005:
            # Santa Catarina do Fogo Municipality Day.
            self._add_holiday_nov_25(tr("Dia do Município de Santa Catarina do Fogo"))

    def _populate_subdiv_cr_public_holidays(self):
        # Law # 93/82 of November 6th.
        if self._year >= 1983:
            # Santa Cruz Municipality Day.
            self._add_holiday_jul_25(tr("Dia do Município de Santa Cruz"))

    def _populate_subdiv_ma_public_holidays(self):
        # Law # 93/82 of November 6th.
        if self._year >= 1983:
            # Maio Municipality Day.
            self._add_holiday_sep_8(tr("Dia do Município do Maio"))

    def _populate_subdiv_mo_public_holidays(self):
        # Law # 23/IV/91 of December 30th.
        if self._year >= 1992:
            # Mosteiros Municipality Day.
            self._add_holiday_aug_15(tr("Dia do Município dos Mosteiros"))

    def _populate_subdiv_pa_public_holidays(self):
        # Law # 93/82 of November 6th.
        if self._year >= 1983:
            # Santo Antão Island Day.
            self._add_holiday_jan_17(tr("Dia da Ilha de Santo Antão"))

            # Paúl Municipality Day.
            self._add_holiday_jun_13(tr("Dia do Município do Paúl"))

    def _populate_subdiv_pn_public_holidays(self):
        # Law # 93/82 of November 6th.
        if self._year >= 1983:
            # Santo Antão Island Day.
            self._add_holiday_jan_17(tr("Dia da Ilha de Santo Antão"))

            # Porto Novo Municipality Day.
            self._add_holiday_sep_2(tr("Dia do Município do Porto Novo"))

    def _populate_subdiv_pr_public_holidays(self):
        # Law # 93/82 of November 6th.
        if self._year >= 1983:
            # Praia Municipality Day.
            self._add_holiday_may_19(tr("Dia do Município da Praia"))

    def _populate_subdiv_rb_public_holidays(self):
        # Law # 67/VI/2005 of May 9th.
        if self._year >= 2005:
            # Ribeira Brava Municipality Day.
            self._add_holiday_dec_6(tr("Dia do Município de Ribeira Brava"))

    def _populate_subdiv_rg_public_holidays(self):
        # Law # 93/82 of November 6th.
        if self._year >= 1983:
            # Santo Antão Island Day.
            self._add_holiday_jan_17(tr("Dia da Ilha de Santo Antão"))

            # Ribeira Grande Municipality Day.
            self._add_holiday_may_7(tr("Dia do Município de Ribeira Grande"))

    def _populate_subdiv_rs_public_holidays(self):
        # Law # 63/VI/2005 of May 9th.
        if self._year >= 2006:
            # Ribeira Grande de Santiago Municipality Day.
            self._add_holiday_jan_31(tr("Dia do Município de Ribeira Grande de Santiago"))

    def _populate_subdiv_sd_public_holidays(self):
        # Law # 96/IV/93 of December 31st.
        if self._year >= 1994:
            # São Domingos Municipality Day.
            self._add_holiday_mar_13(tr("Dia do Município de São Domingos"))

    def _populate_subdiv_sf_public_holidays(self):
        # Law # 23/IV/91 of December 30th.
        if self._year >= 1992:
            # São Filipe Municipality Day.
            self._add_holiday_may_1(tr("Dia do Município de São Filipe"))

    def _populate_subdiv_sl_public_holidays(self):
        # Law # 93/82 of November 6th.
        if self._year >= 1983:
            # Sal Municipality Day.
            self._add_holiday_sep_15(tr("Dia do Município do Sal"))

    def _populate_subdiv_sm_public_holidays(self):
        # Law # 11/V/96 of November 11th.
        if self._year >= 1997:
            # São Miguel Municipality Day.
            self._add_holiday_sep_29(tr("Dia do Município de São Miguel"))

    def _populate_subdiv_so_public_holidays(self):
        # Law # 64/VI/2005 of May 9th.
        if self._year >= 2005:
            # São Lourenço dos Órgãos Municipality Day.
            self._add_holiday_may_9(tr("Dia do Município de São Lourenço dos Órgãos"))

    def _populate_subdiv_ss_public_holidays(self):
        # Law # 65/VI/2005 of May 9th.
        if self._year >= 2005:
            # São Salvador do Mundo Municipality Day.
            self._add_holiday_jul_19(tr("Dia do Município de São Salvador do Mundo"))

    def _populate_subdiv_sv_public_holidays(self):
        # Law # 93/82 of November 6th.
        if self._year >= 1983:
            # São Vicente Municipality Day.
            self._add_holiday_jan_22(tr("Dia do Município de São Vicente"))

            # Carnival Tuesday.
            self._add_carnival_tuesday(tr("Terça-feira de Carnaval"))

    def _populate_subdiv_ta_public_holidays(self):
        # Law # 93/82 of November 6th.
        if self._year >= 1983:
            # Tarrafal de Santiago Municipality Day.
            self._add_holiday_jan_15(tr("Dia do Município do Tarrafal de Santiago"))

    def _populate_subdiv_ts_public_holidays(self):
        # Law # 67/VI/2005 of May 9th.
        if self._year >= 2005:
            # Tarrafal de São Nicolau Municipality Day.
            self._add_holiday_aug_2(tr("Dia do Município do Tarrafal de São Nicolau"))


class CV(CaboVerde):
    pass


class CPV(CaboVerde):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import OCT
from holidays.constants import PUBLIC, WORKDAY
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.holiday_base import HolidayBase


class Slovakia(HolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Slovakia holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Slovakia>
        * <https://sk.wikipedia.org/wiki/Zoznam_sviatkov_na_Slovensku>
        * <https://web.archive.org/web/20250413192257/https://www.slov-lex.sk/pravne-predpisy/SK/ZZ/1993/241/>
        * [Law 326/2020](https://web.archive.org/web/20250713005858/https://static.slov-lex.sk/static/SK/ZZ/2020/326/20210101.html)
        * [Law 530/2023](https://web.archive.org/web/20250424165411/https://static.slov-lex.sk/static/SK/ZZ/2023/530/20240101.html)
        * [Law 261/2025](https://web.archive.org/web/20251014224441/https://static.slov-lex.sk/static/SK/ZZ/2025/261/20251101.html)
    """

    country = "SK"
    default_language = "sk"
    supported_categories = (PUBLIC, WORKDAY)
    supported_languages = ("en_US", "sk", "uk")
    # Independent Slovak Republic established on Jan 01, 1993.
    start_year = 1993

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, cls=SlovakiaStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Day of the Establishment of the Slovak Republic.
        self._add_holiday_jan_1(tr("Deň vzniku Slovenskej republiky"))

        self._add_epiphany_day(
            # Epiphany.
            tr("Zjavenie Pána (Traja králi a vianočný sviatok pravoslávnych kresťanov)")
        )

        # Good Friday.
        self._add_good_friday(tr("Veľký piatok"))

        # Easter Monday.
        self._add_easter_monday(tr("Veľkonočný pondelok"))

        # Labor Day.
        self._add_labor_day(tr("Sviatok práce"))

        if self._year >= 1997 and self._year != 2026:
            # Day of Victory over Fascism.
            self._add_world_war_two_victory_day(tr("Deň víťazstva nad fašizmom"))

        # Saints Cyril and Methodius Day.
        self._add_holiday_jul_5(tr("Sviatok svätého Cyrila a svätého Metoda"))

        # Slovak National Uprising Anniversary.
        self._add_holiday_aug_29(tr("Výročie Slovenského národného povstania"))

        if self._year <= 2023:
            # Constitution Day.
            self._add_holiday_sep_1(tr("Deň Ústavy Slovenskej republiky"))

        if self._year != 2026:
            # Day of Our Lady of the Seven Sorrows.
            self._add_holiday_sep_15(tr("Sedembolestná Panna Mária"))

        # All Saints' Day.
        self._add_all_saints_day(tr("Sviatok Všetkých svätých"))

        if 2001 <= self._year <= 2024:
            # Struggle for Freedom and Democracy Day.
            self._add_holiday_nov_17(tr("Deň boja za slobodu a demokraciu"))

        # Christmas Eve.
        self._add_christmas_eve(tr("Štedrý deň"))

        # Christmas Day.
        self._add_christmas_day(tr("Prvý sviatok vianočný"))

        # Second Day of Christmas.
        self._add_christmas_day_two(tr("Druhý sviatok vianočný"))

    def _populate_workday_holidays(self):
        # According to Law 241/1993, these state holidays are not non-working days.

        # Law 261/2025.
        if self._year == 2026:
            # Day of Victory over Fascism.
            self._add_world_war_two_victory_day(tr("Deň víťazstva nad fašizmom"))

            # Day of Our Lady of the Seven Sorrows.
            self._add_holiday_sep_15(tr("Sedembolestná Panna Mária"))

        # Law 530/2023.
        if self._year >= 2024:
            # Constitution Day.
            self._add_holiday_sep_1(tr("Deň Ústavy Slovenskej republiky"))

        # Law 326/2020.
        if self._year >= 2021:
            # Day of the Establishment of the Independent Czech-Slovak State.
            self._add_holiday_oct_28(tr("Deň vzniku samostatného česko-slovenského štátu"))

        # Law 261/2025.
        if self._year >= 2025:
            # Struggle for Freedom and Democracy Day.
            self._add_holiday_nov_17(tr("Deň boja za slobodu a demokraciu"))


class SK(Slovakia):
    pass


class SVK(Slovakia):
    pass


class SlovakiaStaticHolidays:
    special_public_holidays = {
        # 100th anniversary of the adoption of the Declaration
        # of the Slovak Nation.
        2018: (OCT, 30, tr("100. výročie prijatia Deklarácie slovenského národa"))
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import SEP
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class Niue(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Niue holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Niue>
        * <https://en.wikipedia.org/wiki/Niue_Constitution_Act>
        * [Public Holidays Ordinance 1961](https://web.archive.org/web/20250102100637/http://www.paclii.org/nu/legis/num_act/nipho1961314.pdf)
        * <https://web.archive.org/web/20250223114854/https://niuepocketguide.com/public-holidays-in-niue-other-important-dates/>
        * [2021/2022 Public Holidays](https://web.archive.org/web/20250509105501/https://www.gov.nu/media/pages/information/1018c58017-1725838374/4nov2021-circular.pdf)
        * [2025 Good Friday & Easter Monday and Anzac Day](https://web.archive.org/web/20250719195126/https://www.gov.nu/media/pages/gazette/3e02b3aa84-1746400484/official-psc-circular-easter-anzac-holiday.pdf)
        * [2025 King's Birthday](https://web.archive.org/web/20250719195301/https://www.gov.nu/media/pages/public-service-circulars/3b9f44b6a0-1748573974/king-s-birthday-public-holiday-2025-circular.pdf)
        * <http://archive.today/2025.07.14-145535/https://www.wipo.int/wipolex/en/text/427817>
    """

    country = "NU"
    default_language = "en_NU"
    # %s observed.
    observed_label = tr("%s (observed)")
    # Public Holidays Ordinance 1961.
    start_year = 1962
    supported_languages = ("en_NU", "en_US")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, cls=NiueStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON_TUE)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("New Year's Day")))

        # Takai Commission Holiday.
        self._add_observed(self._add_new_years_day_two(tr("Takai Commission Holiday")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # ANZAC Day.
        self._add_observed(self._add_anzac_day(tr("ANZAC Day")), rule=SAT_SUN_TO_NEXT_MON)

        self._add_holiday_1st_mon_of_jun(
            # King's Birthday.
            tr("King's Birthday")
            if self._year >= 2023
            # Queen's Birthday.
            else tr("Queen's Birthday")
        )

        if self._year >= 1974:
            # Constitution Day.
            self._add_observed(self._add_holiday_oct_19(tr("Constitution Day")))

            # Constitution Day Holiday.
            self._add_observed(self._add_holiday_oct_20(tr("Constitution Day Holiday")))
        else:
            # Annexation Day.
            self._add_holiday_3rd_mon_of_oct(tr("Annexation Day"))

        # Peniamina Gospel Day.
        self._add_holiday_4th_mon_of_oct(tr("Peniamina Gospel Day"))

        # Christmas Day.
        self._add_observed(self._add_christmas_day(tr("Christmas Day")))

        # Boxing Day.
        self._add_observed(self._add_christmas_day_two(tr("Boxing Day")))


class NU(Niue):
    pass


class NIU(Niue):
    pass


class NiueStaticHolidays:
    """Niue special holidays.

    References:
        * [Queen Elizabeth II's Funeral](https://web.archive.org/web/20250617174022/https://tvniue.com/2022/09/premier-will-attend-hm-the-queens-funeral-while-monday-19th-is-declared-one-off-public-holiday/)
    """

    special_public_holidays = {
        # Queen Elizabeth II's Funeral.
        2022: (SEP, 19, tr("Queen Elizabeth II's Funeral")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import JAN, FEB, MAR, SEP, OCT, NOV, DEC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import ObservedHolidayBase, SUN_TO_NEXT_MON


class Namibia(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """Namibia holidays.

    References:
        * [Public Holidays Act, 1990](https://web.archive.org/web/20250322163344/https://namiblii.org/akn/na/act/1990/26/eng@2004-12-17)
        * [Public Holidays Amendment Act, 2004](https://web.archive.org/web/20221015121213/https://namiblii.org/akn/na/act/2004/16/eng@2004-12-17)
        * [Genocide Remembrance Day](https://web.archive.org/web/20250126055207/https://namiblii.org/akn/na/officialGazette/government-gazette/2024-05-28/8373/eng@2024-05-28)

    As of 1991/2/1, whenever a public holiday falls on a Sunday, it rolls over to the Monday,
    unless that Monday is already a public holiday.
    Since the interval from 1991/1/1 to 1991/2/1 includes only New Year's Day, and it's a Tuesday,
    we can assume that the beginning is 1991.
    """

    country = "NA"
    default_language = "en_NA"
    # %s (observed).
    observed_label = tr("%s (observed)")
    start_year = 1991
    supported_languages = ("en_NA", "en_US", "uk")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, NamibiaStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("New Year's Day")))

        # Independence Day.
        self._add_observed(self._add_holiday_mar_21(tr("Independence Day")))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        # Easter Monday.
        self._add_easter_monday(tr("Easter Monday"))

        # Workers' Day.
        self._add_observed(self._add_labor_day(tr("Workers' Day")))

        # Cassinga Day.
        self._add_observed(self._add_holiday_may_4(tr("Cassinga Day")))

        # Africa Day.
        self._add_observed(self._add_africa_day(tr("Africa Day")))

        if self._year >= 2025:
            # Genocide Remembrance Day.
            self._add_holiday_may_28(tr("Genocide Remembrance Day"))

        # Ascension Day.
        self._add_ascension_thursday(tr("Ascension Day"))

        # Heroes' Day.
        self._add_observed(self._add_holiday_aug_26(tr("Heroes' Day")))

        self._add_observed(
            self._add_holiday_sep_10(
                # Day of the Namibian Women and International Human Rights Day.
                tr("Day of the Namibian Women and International Human Rights Day")
                if self._year >= 2005
                # International Human Rights Day.
                else tr("International Human Rights Day")
            )
        )

        # Christmas Day.
        self._add_christmas_day(tr("Christmas Day"))

        # Family Day.
        self._add_observed(self._add_christmas_day_two(tr("Family Day")))


class NA(Namibia):
    pass


class NAM(Namibia):
    pass


class NamibiaStaticHolidays:
    """Namibia special holidays.

    References:
        * [March 1, 1994](https://web.archive.org/web/20241210145818/https://namiblii.org/akn/na/officialGazette/government-gazette/1994-02-15/797/eng@1994-02-15)
        * [December 7, 1994](https://web.archive.org/web/20250613113650/https://namiblii.org/akn/na/officialGazette/government-gazette/1994-11-30/987/eng@1994-11-30)
        * [October 24, 1995](https://web.archive.org/web/20250613184422/https://namiblii.org/akn/na/officialGazette/government-gazette/1995-10-20/1178/eng@1995-10-20)
        * [September 26, 1997](https://web.archive.org/web/20250613172016/https://namiblii.org/akn/na/officialGazette/government-gazette/1997-09-25/1690/eng@1997-09-25)
        * [November 30, 1998](https://web.archive.org/web/20250613184549/https://namiblii.org/akn/na/officialGazette/government-gazette/1998-11-06/1988/eng@1998-11-06)
        * [November 30, 1999](https://web.archive.org/web/20241012184236/https://namiblii.org/akn/na/officialGazette/government-gazette/1999-10-08/2202/eng@1999-10-08)
        * [Y2K changeover](https://web.archive.org/web/20250613145912/https://namiblii.org/akn/na/officialGazette/government-gazette/1999-11-22/2234/eng@1999-11-22)
        * [November 28, 2014](https://web.archive.org/web/20250613184051/https://namiblii.org/akn/na/officialGazette/government-gazette/2014-11-07/5609/eng@2014-11-07)
        * [November 27, 2015](https://web.archive.org/web/20250613184258/https://namiblii.org/akn/na/officialGazette/government-gazette/2015-11-26/5885/eng@2015-11-26)
        * [November 27, 2019](https://web.archive.org/web/20241112233422/https://namiblii.org/akn/na/officialGazette/government-gazette/2019-09-30/7008/eng@2019-09-30)
        * [November 25, 2020](https://web.archive.org/web/20250613112403/https://namiblii.org/akn/na/officialGazette/government-gazette/2020-11-19/7394/eng@2020-11-19)
        * [February 25, 2024](https://web.archive.org/web/20241203060851/https://namiblii.org/akn/na/officialGazette/government-gazette/2024-02-20/8312/eng@2024-02-20)
        * [November 27, 2024](https://web.archive.org/web/20250514020246/https://namiblii.org/akn/na/officialGazette/government-gazette/2024-09-26/8454/eng@2024-09-26)
        * [March 1, 2025](https://web.archive.org/web/20250613172517/https://namiblii.org/akn/na/officialGazette/government-gazette/2025-02-14/8577/eng@2025-02-14)
    """

    # Y2K changeover.
    y2k_changeover = tr("Y2K changeover")

    # General Election Day.
    general_election_day = tr("General Election Day")

    # Regional Election Day.
    regional_election_day = tr("Regional Election Day")

    special_public_holidays = {
        1994: (
            # Walvis Bay Reintegration Day.
            (MAR, 1, tr("Walvis Bay Reintegration Day")),
            (DEC, 7, general_election_day),
        ),
        # Public Holiday.
        1995: (OCT, 24, tr("Public Holiday")),
        # Day of Mourning for Honourable Moses Garoeb.
        1997: (SEP, 26, tr("Day of Mourning for Honourable Moses Garoeb")),
        1998: (NOV, 30, regional_election_day),
        1999: (
            (NOV, 30, general_election_day),
            (DEC, 31, y2k_changeover),
        ),
        2000: (JAN, 3, y2k_changeover),
        2014: (NOV, 28, general_election_day),
        2015: (NOV, 27, regional_election_day),
        2019: (NOV, 27, general_election_day),
        2020: (NOV, 25, regional_election_day),
        2024: (
            # Burial ceremony of Dr. Hage Gottfried Geingob.
            (FEB, 25, tr("Burial ceremony of Dr. Hage Gottfried Geingob")),
            (NOV, 27, general_election_day),
        ),
        # Burial ceremony of Dr. Sam Shafiishuna Nujoma.
        2025: (MAR, 1, tr("Burial ceremony of Dr. Sam Shafiishuna Nujoma")),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from holidays.calendars.gregorian import JAN, FEB, MAR, AUG
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
    SAT_SUN_TO_NEXT_MON,
)


class AntiguaAndBarbuda(
    ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays
):
    """Antigua and Barbuda holidays.

    References:
        * [The Public Holidays Act, 1954](https://web.archive.org/web/20250123110511/https://laws.gov.ag/wp-content/uploads/2018/08/cap-354.pdf)
        * [The Public Holidays (Amendment) Act, 2005](https://web.archive.org/web/20250103221834/https://laws.gov.ag/wp-content/uploads/2018/08/a2005-8.pdf)
        * [The Public Holidays (Amendment) Act, 2014](https://web.archive.org/web/20250427174435/https://laws.gov.ag/wp-content/uploads/2019/03/Public-Holidays-Amendment-Act.pdf)
        * [The Public Holidays (Amendment) Act, 2019](https://web.archive.org/web/20250427174433/https://laws.gov.ag/wp-content/uploads/2020/02/No.-23-of-2019-Public-Holidays-Amendment-Act-2019.pdf)
        * [No. 24 of 2006 Proclamation](https://web.archive.org/web/20240620021627/http://laws.gov.ag/wp-content/uploads/2022/06/No.-24-of-2006-Proclamation-dated-29th-November2006-Appointing-the-11th-of-December2006.pdf)
        * [No. 40 of 2012 Proclamation](https://web.archive.org/web/20250427174546/https://laws.gov.ag/wp-content/uploads/2021/08/No.-40-of-2012-Proclamation-dated-the-27th-day-of-November-2012-Apponting-the-27th-of-December-2012-as-a-Public-Holiday-throughout-Antigua-and-Barbuda.pdf)
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Antigua_and_Barbuda>

    Notes:
        In accordance with No. 24 of 2006 Proclamation, National Heroes Day was celebrated on
        Dec 11, 2006. In accordance with No. 40 of 2012 Proclamation, National Heroes Day was
        celebrated on Dec 10, 2012.
    """

    country = "AG"
    observed_label = "%s (observed)"
    start_year = 1955

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, AntiguaAndBarbudaStaticHolidays)
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day("New Year's Day"))

        # Good Friday.
        self._add_good_friday("Good Friday")

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        # Labour Day.
        self._add_holiday_1st_mon_of_may("Labour Day")

        # Whit Monday.
        self._add_whit_monday("Whit Monday")

        if self._year <= 2005:
            # Caribbean Community (Caricom) Day.
            self._add_holiday_1st_mon_of_jul("Caribbean Community (Caricom) Day")

        # Carnival Monday.
        self._add_holiday_1st_mon_of_aug("Carnival Monday")

        if self._year >= 2006:
            # Carnival Tuesday.
            self._add_holiday_1_day_past_1st_mon_of_aug("Carnival Tuesday")

        # Independence Day.
        dt = self._add_holiday_nov_1("Independence Day")
        if self._year >= 2005:
            self._add_observed(dt, rule=SAT_SUN_TO_NEXT_MON)

        if self._year >= 2005:
            name = (
                # Sir Vere Cornwall Bird Snr. Day.
                "Sir Vere Cornwall Bird Snr. Day"
                if self._year >= 2014
                # National Heroes Day.
                else "National Heroes Day"
            )
            match self._year:
                case 2006:
                    self._add_holiday_dec_11(name)
                case 2012:
                    self._add_holiday_dec_10(name)
                case _:
                    dt = self._add_holiday_dec_9(name)
                    if self._year >= 2020:
                        self._add_observed(dt, rule=SAT_SUN_TO_NEXT_MON)

        # Christmas Day.
        self._add_observed(self._add_christmas_day("Christmas Day"), rule=SAT_SUN_TO_NEXT_MON_TUE)

        # Boxing Day.
        self._add_observed(self._add_christmas_day_two("Boxing Day"), rule=SAT_SUN_TO_NEXT_MON_TUE)


class AG(AntiguaAndBarbuda):
    pass


class ATG(AntiguaAndBarbuda):
    pass


class AntiguaAndBarbudaStaticHolidays:
    """Antigua and Barbuda special holidays.

    References:
        * [August 3, 1993 Holiday](https://web.archive.org/web/20241215194404/https://laws.gov.ag/wp-content/uploads/2021/08/No.-42-of-1993-Proclamation-dated-the-22nd-day-of-July-1993-appointing-Tuesday-the-3rd-day-of-August-1993-as-a-public-holiday-throughout-Antigua-and-Barbuda..pdf)
        * [State Funeral of the late The Honourable Charlesworth T. Samuel](https://web.archive.org/web/20250427174441/https://laws.gov.ag/wp-content/uploads/2022/12/No.-7-of-2008-Proclamation-dated-the-13th-day-of-February-2008-Appointing-Tuesday-19th-February-2008-as-a-Public-Holiday.pdf)
        * [State Funeral of the late The Honourable Sir George Herbert Walter](https://web.archive.org/web/20250427174444/https://laws.gov.ag/wp-content/uploads/2022/12/No.-9-of-2008-Proclamation-dated-the-13th-day-of-March-2008-Appointing-Tuesday-18th-March-2008-as-a-Public-Holiday.pdf)
        * According to the Public Holidays (Amendment) Act, 2014, the day after the general
          election is a holiday.
            * [2018 Antiguan general election](https://en.wikipedia.org/wiki/2018_Antiguan_general_election)
            * [2023 Antiguan general election](https://en.wikipedia.org/wiki/2023_Antiguan_general_election)
            * [The Public Holidays (Amendment) Act, 2014](https://web.archive.org/web/20250427174435/https://laws.gov.ag/wp-content/uploads/2019/03/Public-Holidays-Amendment-Act.pdf)
    """

    # Day after the General Election.
    day_after_the_general_election = "Day after the General Election"

    special_public_holidays = {
        1993: (AUG, 3, "Public Holiday"),
        2008: (
            # State Funeral of the late The Honourable Charlesworth T. Samuel.
            (FEB, 19, "State Funeral of the late The Honourable Charlesworth T. Samuel"),
            # State Funeral of the late The Honourable Sir George Herbert Walter.
            (MAR, 18, "State Funeral of the late The Honourable Sir George Herbert Walter"),
        ),
        2018: (MAR, 22, day_after_the_general_election),
        2023: (JAN, 19, day_after_the_general_election),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars import _CustomChineseHolidays, _CustomIslamicHolidays
from holidays.calendars.gregorian import JAN, FEB, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.constants import PUBLIC, WORKDAY
from holidays.groups import (
    ChineseCalendarHolidays,
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
)
from holidays.holiday_base import HolidayBase


class Philippines(
    HolidayBase,
    ChineseCalendarHolidays,
    ChristianHolidays,
    InternationalHolidays,
    IslamicHolidays,
    StaticHolidays,
):
    """Philippines holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_the_Philippines>
        * [Revised Administrative Code of 1987](https://web.archive.org/web/20241203234427/https://www.officialgazette.gov.ph/1987/07/25/executive-order-no-292-book-ichapter-7-regular-holidays-and-nationwide-special-days/)
        * [Republic Act No. 9177](https://web.archive.org/web/20230930164310/https://www.officialgazette.gov.ph/2002/11/13/republic-act-no-9177/)
        * [Republic Act No. 9256](https://web.archive.org/web/20240706140401/https://www.officialgazette.gov.ph/2004/02/25/republic-act-no-9256/)
        * [Republic Act No. 9492](https://web.archive.org/web/20250413180041/http://www.officialgazette.gov.ph/2007/07/24/republic-act-no-9492/)
        * [Republic Act No. 9645](https://web.archive.org/web/20231014172648/https://www.officialgazette.gov.ph/2009/06/12/republic-act-no-9645/)
        * [Republic Act No. 9849](https://web.archive.org/web/20250424053703/http://officialgazette.gov.ph/2009/12/11/republic-act-no-9849/)
        * [Republic Act No. 10966](https://web.archive.org/web/20250419183417/http://www.officialgazette.gov.ph/2017/12/28/republic-act-no-10966/)
        * [Proclamation No. 944/2020](https://web.archive.org/web/20250428055016/https://www.officialgazette.gov.ph/2020/05/19/proclamation-no-944-s-2020/)
        * [Proclamation No. 985/2020](https://web.archive.org/web/20230901112559/https://www.officialgazette.gov.ph/2020/07/29/proclamation-no-985-s-2020/)
        * [Proclamation No. 90/2022](https://web.archive.org/web/20231026052921/https://www.officialgazette.gov.ph/2022/11/09/proclamation-no-90-s-2022/)
        * [Proclamation No. 665/2024](https://archive.org/details/20241015-proc-665-frm)
        * [Proclamation No. 729/2024](https://archive.org/details/20241030-proc-729-frm)
        * [Nationwide holidays 2018-2025](https://web.archive.org/web/20240515022447/https://www.officialgazette.gov.ph/nationwide-holidays/2018/)
        * [Proclamation No. 839/2025](https://archive.org/details/20250320-proc-839-frm_202506)
        * [Proclamation No. 878/2025](https://archive.org/details/20250506-proc-878-frm_202506)
        * [Proclamation No. 911/2025](https://archive.org/details/20250521-proc-911-frm_20250606_1800)
        * [Proclamation No. 1006/2025](https://archive.org/details/20250903-proc-1006-frm)
    """

    country = "PH"
    supported_categories = (PUBLIC, WORKDAY)
    default_language = "en_PH"
    # %s (estimated).
    estimated_label = tr("%s (estimated)")
    supported_languages = ("en_PH", "en_US", "fil", "th")
    start_year = 1988

    def __init__(self, *args, islamic_show_estimated: bool = True, **kwargs):
        """
        Args:
            islamic_show_estimated:
                Whether to add "estimated" label to Islamic holidays name
                if holiday date is estimated.
        """
        ChineseCalendarHolidays.__init__(self, cls=PhilippinesChineseHolidays)
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        IslamicHolidays.__init__(
            self, cls=PhilippinesIslamicHolidays, show_estimated=islamic_show_estimated
        )
        StaticHolidays.__init__(self, cls=PhilippinesStaticHolidays)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_new_years_day(tr("New Year's Day"))

        if self._year >= 2012 and self._year != 2023:
            # Chinese New Year.
            self._add_chinese_new_years_day(tr("Chinese New Year"))

        if 2016 <= self._year <= 2023 and self._year != 2017:
            dates_obs = {
                2023: (FEB, 24),
            }
            self._add_holiday(
                # EDSA People Power Revolution Anniversary.
                tr("EDSA People Power Revolution Anniversary"),
                dates_obs.get(self._year, (FEB, 25)),
            )

        # Maundy Thursday.
        self._add_holy_thursday(tr("Maundy Thursday"))

        # Good Friday.
        self._add_good_friday(tr("Good Friday"))

        if self._year >= 2013:
            # Black Saturday.
            self._add_holy_saturday(tr("Black Saturday"))

        dates_obs = {
            2008: (APR, 7),
            2009: (APR, 6),
            2023: (APR, 10),
        }
        # Day of Valor.
        self._add_holiday(tr("Araw ng Kagitingan"), dates_obs.get(self._year, (APR, 9)))

        # Labor Day.
        self._add_labor_day(tr("Labor Day"))

        dates_obs = {
            2007: (JUN, 11),
            2008: (JUN, 9),
            2010: (JUN, 14),
        }
        # Independence Day.
        self._add_holiday(tr("Independence Day"), dates_obs.get(self._year, (JUN, 12)))

        if self._year >= 2004:
            dates_obs = {
                2007: (AUG, 20),
                2008: (AUG, 18),
                2010: (AUG, 23),
                2024: (AUG, 23),
            }
            # Ninoy Aquino Day.
            self._add_holiday(tr("Ninoy Aquino Day"), dates_obs.get(self._year, (AUG, 21)))

        # National Heroes Day.
        name = tr("National Heroes Day")
        if self._year >= 2007:
            self._add_holiday_last_mon_of_aug(name)
        else:
            self._add_holiday_last_sun_of_aug(name)

        # All Saints' Day.
        self._add_all_saints_day(tr("All Saints' Day"))

        dates_obs = {
            2008: (DEC, 1),
            2010: (NOV, 29),
            2023: (NOV, 27),
        }
        # Bonifacio Day.
        self._add_holiday(tr("Bonifacio Day"), dates_obs.get(self._year, (NOV, 30)))

        if self._year >= 2019:
            # Immaculate Conception.
            self._add_immaculate_conception_day(tr("Feast of the Immaculate Conception of Mary"))

        # Christmas Day.
        self._add_christmas_day(tr("Christmas Day"))

        dates_obs = {
            2010: (DEC, 27),
        }
        # Rizal Day.
        self._add_holiday(tr("Rizal Day"), dates_obs.get(self._year, (DEC, 30)))

        if self._year not in {2021, 2022}:
            # New Year's Eve.
            self._add_new_years_eve(tr("Last Day of the Year"))

        if self._year >= 2002:
            # Eid al-Fitr.
            self._add_eid_al_fitr_day(tr("Eid'l Fitr"))

        if self._year >= 2010:
            # Eid al-Adha.
            self._add_eid_al_adha_day(tr("Eid'l Adha"))

    def _populate_workday_holidays(self):
        # Added in 2009, get special non-working day status in 2025:
        if self._year >= 2009 and self._year != 2025:
            # Founding Anniversary of Iglesia ni Cristo.
            self._add_holiday_jul_27(tr("Founding Anniversary of Iglesia ni Cristo"))

        # Added from 2025 onwards as first decreed in
        # https://web.archive.org/web/20250326064645/https://www.officialgazette.gov.ph/downloads/2024/10oct/20241030-PROC-727-FRM.pdf
        if self._year >= 2025:
            # EDSA People Power Revolution Anniversary.
            self._add_holiday_feb_25(tr("EDSA People Power Revolution Anniversary"))


class PH(Philippines):
    pass


class PHL(Philippines):
    pass


class PhilippinesChineseHolidays(_CustomChineseHolidays):
    LUNAR_NEW_YEAR_DATES_CONFIRMED_YEARS = (2012, 2025)


class PhilippinesIslamicHolidays(_CustomIslamicHolidays):
    EID_AL_ADHA_DATES_CONFIRMED_YEARS = (2010, 2025)
    EID_AL_ADHA_DATES = {
        2010: (NOV, 17),
        2011: (NOV, 7),
        2014: (OCT, 6),
        2015: (SEP, 25),
        2016: (SEP, 10),
        2017: (SEP, 2),
        2019: (AUG, 12),
        2024: (JUN, 17),
    }

    EID_AL_FITR_DATES_CONFIRMED_YEARS = (2002, 2025)
    EID_AL_FITR_DATES = {
        2002: (DEC, 6),
        2003: (NOV, 26),
        2005: (NOV, 4),
        2006: (OCT, 24),
        2007: (OCT, 12),
        2009: (SEP, 21),
        2012: (AUG, 20),
        2013: (AUG, 9),
        2014: (JUL, 29),
        2016: (JUL, 7),
        2017: (JUN, 26),
        2019: (JUN, 5),
        2020: (MAY, 25),
        2022: (MAY, 3),
        2025: (APR, 1),
    }


class PhilippinesStaticHolidays:
    # Additional special (non-working) day.
    additional_special = tr("Additional special (non-working) day")

    # Elections special (non-working) day.
    election_special = tr("Elections special (non-working) day")

    # Christmas Eve.
    christmas_eve = tr("Christmas Eve")

    special_public_holidays = {
        2008: (
            (DEC, 26, additional_special),
            (DEC, 29, additional_special),
        ),
        2009: (
            (NOV, 2, additional_special),
            (DEC, 24, additional_special),
        ),
        2010: (DEC, 24, additional_special),
        2012: (NOV, 2, additional_special),
        2013: (
            (NOV, 2, additional_special),
            (DEC, 24, additional_special),
        ),
        2014: (
            (DEC, 24, additional_special),
            (DEC, 26, additional_special),
        ),
        2015: (
            (JAN, 2, additional_special),
            (DEC, 24, additional_special),
        ),
        2016: (
            (JAN, 2, additional_special),
            (OCT, 31, additional_special),
            (DEC, 24, additional_special),
        ),
        2017: (
            (JAN, 2, additional_special),
            (OCT, 31, additional_special),
        ),
        2018: (
            (MAY, 14, election_special),
            (NOV, 2, additional_special),
            (DEC, 24, additional_special),
        ),
        2019: (
            (MAY, 13, election_special),
            (NOV, 2, additional_special),
            (DEC, 24, additional_special),
        ),
        2020: (
            (NOV, 2, additional_special),
            (DEC, 24, additional_special),
        ),
        2022: (
            (MAY, 9, election_special),
            (OCT, 31, additional_special),
        ),
        2023: (
            (JAN, 2, additional_special),
            (OCT, 30, election_special),
            (NOV, 2, additional_special),
            (DEC, 26, additional_special),
        ),
        2024: (
            (FEB, 9, additional_special),
            (NOV, 2, additional_special),
            (DEC, 24, additional_special),
        ),
        2025: (
            (MAY, 12, election_special),
            (JUL, 27, additional_special),
            # All Saints' Day Eve.
            (OCT, 31, tr("All Saints' Day Eve")),
            (DEC, 24, christmas_eve),
        ),
        2026: (
            # All Souls' Day.
            (NOV, 2, tr("All Souls' Day")),
            (DEC, 24, christmas_eve),
        ),
    }
